@@ -3,43 +3,46 @@ NULL
 
 #' Wrap a 1d ribbon of panels into 2d
 #'
-#' \code{facet_wrap} wraps a 1d sequence of panels into 2d. This is generally
-#' a better use of screen space than \code{\link{facet_grid}} because most
+#' `facet_wrap` wraps a 1d sequence of panels into 2d. This is generally
+#' a better use of screen space than [facet_grid()] because most
 #' displays are roughly rectangular.
 #'
-#' @param facets Either a formula or character vector. Use either a
-#'   one sided formula, \code{~a + b}, or a character vector, \code{c("a", "b")}.
+#' @param facets A set of variables or expressions quoted by [vars()]
+#'   and defining faceting groups on the rows or columns dimension.
+#'   The variables can be named (the names are passed to `labeller`).
+#'
+#'   For compatibility with the classic interface, can also be a
+#'   formula or character vector. Use either a one sided formula, `~a
+#'   + b`, or a character vector, `c("a", "b")`.
 #' @param nrow,ncol Number of rows and columns.
-#' @param scales should Scales be fixed (\code{"fixed"}, the default),
-#'   free (\code{"free"}), or free in one dimension (\code{"free_x"},
-#'   \code{"free_y"}).
+#' @param scales Should scales be fixed (`"fixed"`, the default),
+#'   free (`"free"`), or free in one dimension (`"free_x"`,
+#'   `"free_y"`)?
 #' @param strip.position By default, the labels are displayed on the top of
-#'   the plot. Using \code{strip.position} it is possible to place the labels on
+#'   the plot. Using `strip.position` it is possible to place the labels on
 #'   either of the four sides by setting \code{strip.position = c("top",
 #'   "bottom", "left", "right")}
-#' @param dir Direction: either "h" for horizontal, the default, or "v", for
-#'   vertical.
+#' @param dir Direction: either `"h"` for horizontal, the default, or `"v"`,
+#'   for vertical.
 #' @inheritParams facet_grid
 #' @export
 #' @examples
-#' ggplot(mpg, aes(displ, hwy)) +
-#'   geom_point() +
-#'   facet_wrap(~class)
+#' p <- ggplot(mpg, aes(displ, hwy)) + geom_point()
+#'
+#' # Use vars() to supply faceting variables:
+#' p + facet_wrap(vars(class))
+#'
+#' # The historical interface with formulas is also available:
+#' p + facet_wrap(~class)
 #'
 #' # Control the number of rows and columns with nrow and ncol
-#' ggplot(mpg, aes(displ, hwy)) +
-#'   geom_point() +
-#'   facet_wrap(~class, nrow = 4)
+#' p + facet_wrap(vars(class), nrow = 4)
 #'
 #' \donttest{
 #' # You can facet by multiple variables
 #' ggplot(mpg, aes(displ, hwy)) +
 #'   geom_point() +
-#'   facet_wrap(~ cyl + drv)
-#' # Or use a character vector:
-#' ggplot(mpg, aes(displ, hwy)) +
-#'   geom_point() +
-#'   facet_wrap(c("cyl", "drv"))
+#'   facet_wrap(vars(cyl, drv))
 #'
 #' # Use the `labeller` option to control how labels are printed:
 #' ggplot(mpg, aes(displ, hwy)) +
@@ -62,7 +65,7 @@ NULL
 #'   facet_wrap(~class, scales = "free")
 #'
 #' # To repeat the same data in every panel, simply construct a data frame
-#' # that does not contain the facetting variable.
+#' # that does not contain the faceting variable.
 #' ggplot(mpg, aes(displ, hwy)) +
 #'   geom_point(data = transform(mpg, class = NULL), colour = "grey85") +
 #'   geom_point() +
@@ -105,13 +108,23 @@ facet_wrap <- function(facets, nrow = NULL, ncol = NULL, scales = "fixed",
   # Check for deprecated labellers
   labeller <- check_labeller(labeller)
 
+  # Flatten all facets dimensions into a single one
+  facets_list <- as_facets_list(facets)
+  facets <- rlang::flatten_if(facets_list, rlang::is_list)
+
   ggproto(NULL, FacetWrap,
     shrink = shrink,
-    params = list(facets = as.quoted(facets), free = free,
-    as.table = as.table, strip.position = strip.position,
-    drop = drop, ncol = ncol, nrow = nrow,
-    labeller = labeller,
-    dir = dir)
+    params = list(
+      facets = facets,
+      free = free,
+      as.table = as.table,
+      strip.position = strip.position,
+      drop = drop,
+      ncol = ncol,
+      nrow = nrow,
+      labeller = labeller,
+      dir = dir
+    )
   )
 }
 
@@ -123,8 +136,10 @@ FacetWrap <- ggproto("FacetWrap", Facet,
   shrink = TRUE,
 
   compute_layout = function(data, params) {
-    vars <- as.quoted(params$facets)
-    if (length(vars) == 0) return(layout_null())
+    vars <- params$facets
+    if (length(vars) == 0) {
+      return(layout_null())
+    }
 
     base <- plyr::unrowname(
       combine_vars(data, params$plot_env, vars, drop = params$drop)
@@ -162,9 +177,9 @@ FacetWrap <- ggproto("FacetWrap", Facet,
     if (empty(data)) {
       return(cbind(data, PANEL = integer(0)))
     }
-    vars <- as.quoted(params$facets)
+    vars <- params$facets
 
-    facet_vals <- eval_facet_vars(vars, data, params$plot_env)
+    facet_vals <- eval_facets(vars, data, params$plot_env)
     facet_vals[] <- lapply(facet_vals[], as.factor)
 
     missing_facets <- setdiff(names(vars), names(facet_vals))
@@ -187,11 +202,10 @@ FacetWrap <- ggproto("FacetWrap", Facet,
     data[order(data$PANEL), ]
   },
   draw_panels = function(panels, layout, x_scales, y_scales, ranges, coord, data, theme, params) {
-    # If coord is non-cartesian and (x is free or y is free)
-    # then throw error
-    if ((!inherits(coord, "CoordCartesian")) && (params$free$x || params$free$y)) {
-      stop("ggplot2 does not currently support free scales with a non-cartesian coord", call. = FALSE)
+    if ((params$free$x || params$free$y) && !coord$is_free()) {
+      stop(snake_class(coord), " doesn't support free scales", call. = FALSE)
     }
+
     if (inherits(coord, "CoordFlip")) {
       if (params$free$x) {
         layout$SCALE_X <- seq_len(nrow(layout))
@@ -242,7 +256,7 @@ FacetWrap <- ggproto("FacetWrap", Facet,
     empties <- apply(panel_table, c(1,2), function(x) is.zero(x[[1]]))
     panel_table <- gtable_matrix("layout", panel_table,
      widths = unit(rep(1, ncol), "null"),
-     heights = unit(rep(aspect_ratio, nrow), "null"), respect = respect, clip = "on", z = matrix(1, ncol = ncol, nrow = nrow))
+     heights = unit(rep(aspect_ratio, nrow), "null"), respect = respect, clip = coord$clip, z = matrix(1, ncol = ncol, nrow = nrow))
     panel_table$layout$name <- paste0('panel-', rep(seq_len(ncol), nrow), '-', rep(seq_len(nrow), each = ncol))
 
     panel_table <- gtable_add_col_space(panel_table,
@@ -283,14 +297,16 @@ FacetWrap <- ggproto("FacetWrap", Facet,
       col_axes <- axes$y$right[layout$SCALE_Y[col_panels]]
       if (params$strip.position == "bottom" &&
           theme$strip.placement != "inside" &&
-          any(!vapply(row_axes, is.zero, logical(length(row_axes))))) {
+          any(!vapply(row_axes, is.zero, logical(1))) &&
+          !params$free$x) {
         warning("Suppressing axis rendering when strip.position = 'bottom' and strip.placement == 'outside'", call. = FALSE)
       } else {
         axis_mat_x_bottom[row_pos] <- row_axes
       }
       if (params$strip.position == "right" &&
           theme$strip.placement != "inside" &&
-          any(!vapply(col_axes, is.zero, logical(length(col_axes))))) {
+          any(!vapply(col_axes, is.zero, logical(1))) &&
+          !params$free$y) {
         warning("Suppressing axis rendering when strip.position = 'right' and strip.placement == 'outside'", call. = FALSE)
       } else {
         axis_mat_y_right[col_pos] <- col_axes
@@ -315,7 +331,7 @@ FacetWrap <- ggproto("FacetWrap", Facet,
         strip_pad <- axis_height_bottom
       }
       strip_height <- unit(apply(strip_mat, 1, max_height), "cm")
-      panel_table <- weave_tables_row(panel_table, strip_mat, placement, strip_height, strip_name, 2, "on")
+      panel_table <- weave_tables_row(panel_table, strip_mat, placement, strip_height, strip_name, 2, coord$clip)
       if (!inside) {
         strip_pad[unclass(strip_pad) != 0] <- strip_padding
         panel_table <- weave_tables_row(panel_table, row_shift = placement, row_height = strip_pad)
@@ -331,13 +347,16 @@ FacetWrap <- ggproto("FacetWrap", Facet,
       }
       strip_pad[unclass(strip_pad) != 0] <- strip_padding
       strip_width <- unit(apply(strip_mat, 2, max_width), "cm")
-      panel_table <- weave_tables_col(panel_table, strip_mat, placement, strip_width, strip_name, 2, "on")
+      panel_table <- weave_tables_col(panel_table, strip_mat, placement, strip_width, strip_name, 2, coord$clip)
       if (!inside) {
         strip_pad[unclass(strip_pad) != 0] <- strip_padding
         panel_table <- weave_tables_col(panel_table, col_shift = placement, col_width = strip_pad)
       }
     }
     panel_table
+  },
+  vars = function(self) {
+    names(self$params$facets)
   }
 )
 
@@ -347,15 +366,15 @@ FacetWrap <- ggproto("FacetWrap", Facet,
 #' Sanitise the number of rows or columns
 #'
 #' Cleans up the input to be an integer greater than or equal to one, or
-#' \code{NULL}. Intended to be used on the \code{nrow} and \code{ncol}
-#' arguments of \code{facet_wrap}.
-#' @param n Hopefully an integer greater than or equal to one, or \code{NULL},
+#' `NULL`. Intended to be used on the `nrow` and `ncol`
+#' arguments of `facet_wrap`.
+#' @param n Hopefully an integer greater than or equal to one, or `NULL`,
 #' though other inputs are handled.
-#' @return An integer greater than or equal to one, or \code{NULL}.
+#' @return An integer greater than or equal to one, or `NULL`.
 #' @note If the length of the input is greater than one, only the first element
 #' is returned, with a warning.
 #' If the input is not an integer, it will be coerced to be one.
-#' If the value is less than one, \code{NULL} is returned, effectively ignoring
+#' If the value is less than one, `NULL` is returned, effectively ignoring
 #' the argument.
 #' Multiple warnings may be generated.
 #' @examples
