@@ -1,11 +1,11 @@
 #' @section Scales:
 #'
-#' All \code{scale_*} functions (like \code{scale_x_continuous}) return a
-#' \code{Scale*} object (like \code{ScaleContinuous}). The \code{Scale*}
+#' All `scale_*` functions (like `scale_x_continuous`) return a
+#' `Scale*` object (like `ScaleContinuous`). The `Scale*`
 #' object represents a single scale.
 #'
-#' Each of the \code{Scale*} objects is a \code{\link{ggproto}} object,
-#' descended from the top-level \code{Scale}.
+#' Each of the `Scale*` objects is a [ggproto()] object,
+#' descended from the top-level `Scale`.
 #'
 #' @rdname ggplot2-ggproto
 #' @format NULL
@@ -30,6 +30,7 @@ Scale <- ggproto("Scale", NULL,
   breaks = waiver(),
   labels = waiver(),
   guide = "legend",
+  position = "left",
 
 
   is_discrete = function() {
@@ -39,7 +40,7 @@ Scale <- ggproto("Scale", NULL,
   # Train scale from a data frame.
   #
   # @return updated range (invisibly)
-  # @seealso \code{\link{scale_train}} for scale specific generic method
+  # @seealso [scale_train()] for scale specific generic method
   train_df = function(self, df) {
     if (empty(df)) return()
 
@@ -120,9 +121,9 @@ Scale <- ggproto("Scale", NULL,
   },
 
   # The physical size of the scale.
-  # This always returns a numeric vector of length 2, giving the physical
+  # This always returns a numeric vector of length 4, giving the physical
   # dimensions of a scale.
-  dimension = function(self, expand = c(0, 0)) {
+  dimension = function(self, expand = c(0, 0, 0, 0)) {
     stop("Not implemented", call. = FALSE)
   },
 
@@ -151,6 +152,23 @@ Scale <- ggproto("Scale", NULL,
 
   break_info = function(self, range = NULL) {
     stop("Not implemented", call. = FALSE)
+  },
+
+  # Only relevant for positional scales
+  axis_order = function(self) {
+    ord <- c("primary", "secondary")
+    if (self$position %in% c("right", "bottom")) {
+      ord <- rev(ord)
+    }
+    ord
+  },
+
+  # Here to make it possible for scales to modify the default titles
+  make_title = function(title) {
+    title
+  },
+  make_sec_title = function(title) {
+    title
   }
 )
 
@@ -187,11 +205,17 @@ ScaleContinuous <- ggproto("ScaleContinuous", Scale,
   },
 
   transform = function(self, x) {
-     self$trans$transform(x)
+     new_x <- self$trans$transform(x)
+     if (any(is.finite(x) != is.finite(new_x))) {
+       type <- if (self$scale_name == "position_c") "continuous" else "discrete"
+       axis <- if ("x" %in% self$aesthetics) "x" else "y"
+       warning("Transformation introduced infinite values in ", type, " ", axis, "-axis", call. = FALSE)
+     }
+     new_x
   },
 
   map = function(self, x, limits = self$get_limits()) {
-    x <- self$oob(self$rescaler(x, from = limits))
+    x <- self$rescaler(self$oob(x, range = limits), from = limits)
 
     uniq <- unique(x)
     pal <- self$palette(uniq)
@@ -200,8 +224,8 @@ ScaleContinuous <- ggproto("ScaleContinuous", Scale,
     ifelse(!is.na(scaled), scaled, self$na.value)
   },
 
-  dimension = function(self, expand = c(0, 0)) {
-    expand_range(self$get_limits(), expand[1], expand[2])
+  dimension = function(self, expand = c(0, 0, 0, 0)) {
+    expand_range4(self$get_limits(), expand)
   },
 
   get_breaks = function(self, limits = self$get_limits()) {
@@ -232,10 +256,6 @@ ScaleContinuous <- ggproto("ScaleContinuous", Scale,
     #       guides cannot discriminate oob from missing value.
     breaks <- censor(self$trans$transform(breaks), self$trans$transform(limits),
                      only.finite = FALSE)
-    if (length(breaks) == 0) {
-      stop("Zero breaks in scale for ", paste(self$aesthetics, collapse = "/"),
-        call. = FALSE)
-    }
     breaks
   },
 
@@ -252,14 +272,7 @@ ScaleContinuous <- ggproto("ScaleContinuous", Scale,
       if (is.null(b)) {
         breaks <- NULL
       } else {
-        b <- b[!is.na(b)]
-        if (length(b) < 2) return()
-
-        bd <- diff(b)[1]
-        if (min(limits) < min(b)) b <- c(b[1] - bd, b)
-        if (max(limits) > max(b)) b <- c(b, b[length(b)] + bd)
-        breaks <- unique(unlist(mapply(seq, b[-length(b)], b[-1], length.out = n + 1,
-          SIMPLIFY = FALSE)))
+        breaks <- self$trans$minor_breaks(b, limits, n)
       }
     } else if (is.function(self$minor_breaks)) {
       # Find breaks in data space, and convert to numeric
@@ -345,12 +358,14 @@ ScaleContinuous <- ggproto("ScaleContinuous", Scale,
 ScaleDiscrete <- ggproto("ScaleDiscrete", Scale,
   drop = TRUE,
   na.value = NA,
+  n.breaks.cache = NULL,
+  palette.cache = NULL,
 
   is_discrete = function() TRUE,
 
   train = function(self, x) {
     if (length(x) == 0) return()
-    self$range$train(x, drop = self$drop)
+    self$range$train(x, drop = self$drop, na.rm = !self$na.translate)
   },
 
   transform = function(x) {
@@ -359,7 +374,14 @@ ScaleDiscrete <- ggproto("ScaleDiscrete", Scale,
 
   map = function(self, x, limits = self$get_limits()) {
     n <- sum(!is.na(limits))
-    pal <- self$palette(n)
+    if (!is.null(self$n.breaks.cache) && self$n.breaks.cache == n) {
+      pal <- self$palette.cache
+    } else {
+      if (!is.null(self$n.breaks.cache)) warning("Cached palette does not match requested", call. = FALSE)
+      pal <- self$palette(n)
+      self$palette.cache <- pal
+      self$n.breaks.cache <- n
+    }
 
     if (is.null(names(pal))) {
       pal_match <- pal[match(as.character(x), limits)]
@@ -368,11 +390,15 @@ ScaleDiscrete <- ggproto("ScaleDiscrete", Scale,
       pal_match <- unname(pal_match)
     }
 
-    ifelse(is.na(x) | is.na(pal_match), self$na.value, pal_match)
+    if (self$na.translate) {
+      ifelse(is.na(x) | is.na(pal_match), self$na.value, pal_match)
+    } else {
+      pal_match
+    }
   },
 
-  dimension = function(self, expand = c(0, 0)) {
-    expand_range(length(self$get_limits()), expand[1], expand[2])
+  dimension = function(self, expand = c(0, 0, 0, 0)) {
+    expand_range4(length(self$get_limits()), expand)
   },
 
   get_breaks = function(self, limits = self$get_limits()) {
@@ -406,8 +432,14 @@ ScaleDiscrete <- ggproto("ScaleDiscrete", Scale,
       return(NULL)
     } else if (identical(self$labels, NA)) {
       stop("Invalid labels specification. Use NULL, not NA", call. = FALSE)
-    }else if (is.waive(self$labels)) {
-      format(self$get_breaks(), justify = "none", trim = TRUE)
+    } else if (is.waive(self$labels)) {
+      breaks <- self$get_breaks()
+      if (is.numeric(breaks)) {
+        # Only format numbers, because on Windows, format messes up encoding
+        format(breaks, justify = "none")
+      } else {
+        as.character(breaks)
+      }
     } else if (is.function(self$labels)) {
       self$labels(breaks)
     } else {
@@ -465,40 +497,43 @@ ScaleDiscrete <- ggproto("ScaleDiscrete", Scale,
 #' Continuous scale constructor.
 #'
 #' @export
-#' @inheritParams discrete_scale
+#' @param aesthetics The names of the aesthetics that this scale works with
+#' @param scale_name The name of the scale
+#' @param palette A palette function that when called with a single integer
+#'   argument (the number of levels in the scale) returns the values that
+#'   they should take
 #' @param name The name of the scale. Used as axis or legend title. If
-#'   \code{NULL}, the default, the name of the scale is taken from the first
-#'   mapping used for that aesthetic.
-#' @param breaks One of: \itemize{
-#'   \item \code{NULL} for no breaks
-#'   \item \code{waiver()} for the default breaks computed by the
+#'   `waiver()`, the default, the name of the scale is taken from the first
+#'   mapping used for that aesthetic. If `NULL`, the legend title will be
+#'   omitted.
+#' @param breaks One of:
+#'   - `NULL` for no breaks
+#'   - `waiver()` for the default breaks computed by the
 #'     transformation object
-#'   \item A numeric vector of positions
-#'   \item A function that takes the limits as input and returns breaks
+#'   - A numeric vector of positions
+#'   - A function that takes the limits as input and returns breaks
 #'     as output
-#' }
-#' @param minor_breaks One of: \itemize{
-#'   \item \code{NULL} for no minor breaks
-#'   \item \code{waiver()} for the default breaks (one minor break between
+#' @param minor_breaks One of:
+#'   - `NULL` for no minor breaks
+#'   - `waiver()` for the default breaks (one minor break between
 #'     each major break)
-#'   \item A numeric vector of positions
-#'   \item A function that given the limits returns a vector of minor breaks.
-#' }
-#' @param labels One of: \itemize{
-#'   \item \code{NULL} for no labels
-#'   \item \code{waiver()} for the default labels computed by the
+#'   - A numeric vector of positions
+#'   - A function that given the limits returns a vector of minor breaks.
+#' @param labels One of:
+#'   - `NULL` for no labels
+#'   - `waiver()` for the default labels computed by the
 #'     transformation object
-#'   \item A character vector giving labels (must be same length as \code{breaks})
-#'   \item A function that takes the breaks as input and returns labels
+#'   - A character vector giving labels (must be same length as `breaks`)
+#'   - A function that takes the breaks as input and returns labels
 #'     as output
-#' }
 #' @param limits A numeric vector of length two providing limits of the scale.
-#'   Use \code{NA} to refer to the existing minimum or maximum.
+#'   Use `NA` to refer to the existing minimum or maximum.
 #' @param rescaler  Used by diverging and n colour gradients
-#'   (i.e. \code{\link{scale_colour_gradient2}}, \code{\link{scale_colour_gradientn}}).
-#'   A function used to scale the input values to the range [0, 1].
+#'   (i.e. [scale_colour_gradient2()], [scale_colour_gradientn()]).
+#'   A function used to scale the input values to the range \eqn{[0, 1]}.
 #' @param oob Function that handles limits outside of the scale limits
 #'   (out of bounds). The default replaces out of bounds values with NA.
+#' @inheritParams scale_x_discrete
 #' @param na.value Missing values will be replaced with this value.
 #' @param trans Either the name of a transformation object, or the
 #'   object itself. Built-in transformations include "asn", "atanh",
@@ -507,25 +542,24 @@ ScaleDiscrete <- ggproto("ScaleDiscrete", Scale,
 #'
 #'   A transformation object bundles together a transform, it's inverse,
 #'   and methods for generating breaks and labels. Transformation objects
-#'   are defined in the scales package, and are called \code{name_trans}, e.g.
-#'   \code{\link[scales]{boxcox_trans}}. You can create your own
-#'   transformation with \code{\link[scales]{trans_new}}.
-#' @param expand A numeric vector of length two giving multiplicative and
-#'   additive expansion constants. These constants ensure that the data is
-#'   placed some distance away from the axes. The defaults are
-#'   \code{c(0.05, 0)} for continuous variables, and \code{c(0, 0.6)} for
-#'   discrete variables.
-#' @param guide Name of guide object, or object itself.
+#'   are defined in the scales package, and are called `name_trans`, e.g.
+#'   [scales::boxcox_trans()]. You can create your own
+#'   transformation with [scales::trans_new()].
+#' @param guide A function used to create a guide or its name. See
+#'   [guides()] for more info.
+#' @param position The position of the axis. "left" or "right" for vertical
+#' scales, "top" or "bottom" for horizontal scales
+#' @param super The super class to use for the constructed scale
 #' @keywords internal
 continuous_scale <- function(aesthetics, scale_name, palette, name = waiver(),
-                             breaks = waiver(), minor_breaks = waiver(),
-                             labels = waiver(), limits = NULL,
-                             rescaler = rescale, oob = censor,
-                             expand = waiver(), na.value = NA_real_,
-                             trans = "identity", guide = "legend") {
+  breaks = waiver(), minor_breaks = waiver(), labels = waiver(), limits = NULL,
+  rescaler = rescale, oob = censor, expand = waiver(), na.value = NA_real_,
+  trans = "identity", guide = "legend", position = "left", super = ScaleContinuous) {
 
   check_breaks_labels(breaks, labels)
 
+  position <- match.arg(position, c("left", "right", "top", "bottom"))
+
   if (is.null(breaks) && !is_position_aes(aesthetics) && guide != "none") {
     guide <- "none"
   }
@@ -535,7 +569,7 @@ continuous_scale <- function(aesthetics, scale_name, palette, name = waiver(),
     limits <- trans$transform(limits)
   }
 
-  ggproto(NULL, ScaleContinuous,
+  ggproto(NULL, super,
     call = match.call(),
 
     aesthetics = aesthetics,
@@ -555,62 +589,48 @@ continuous_scale <- function(aesthetics, scale_name, palette, name = waiver(),
     minor_breaks = minor_breaks,
 
     labels = labels,
-    guide = guide
+    guide = guide,
+    position = position
   )
 }
 
 #' Discrete scale constructor.
 #'
 #' @export
-#' @param aesthetics the names of the aesthetics that this scale works with
-#' @param scale_name the name of the scale
-#' @param palette a palette function that when called with a single integer
-#'   argument (the number of levels in the scale) returns the values that
-#'   they should take
-#' @param name the name of the scale - used as the axis label or the legend
-#'  title
+#' @inheritParams continuous_scale
+#' @param breaks One of:
+#'   - `NULL` for no breaks
+#'   - `waiver()` for the default breaks computed by the
+#'     transformation object
+#'   - A character vector of breaks
+#'   - A function that takes the limits as input and returns breaks
+#'     as output
+#' @param limits A character vector that defines possible values of the scale
+#'   and their order.
 #' @param drop Should unused factor levels be omitted from the scale?
-#'    The default, \code{TRUE}, uses the levels that appear in the data;
-#'    \code{FALSE} uses all the levels in the factor.
-#' @param breaks control the breaks in the guide.  There are four possible
-#'   types of input:
-#'   \itemize{
-#'     \item \code{NULL}: don't display any breaks
-#'     \item a character vector giving the breaks as they should appear on the
-#'      axis or in the legend.
-#'     \item \code{waiver()} to use the default break computation.
-#'     \item a function, that when called with a single argument, a character
-#'       vector giving the limits of the scale, returns a character vector
-#'       specifying which breaks to display.
-#'   }
-#'   This parameter does not affect in any way how the data is scaled - it
-#'   only affects the appearance of the legend.
-#' @param limits A character vector specifying the data range for the scale.
-#   The limits control what levels are displayed in the plot, their order,
-#'  and the default order of their display in guides.
-#' @param labels \code{NULL} for no labels, \code{waiver()} for default
-#'   labels (labels the same as breaks), a character vector the same length
-#'   as breaks, or a named character vector whose names are used to match
-#'   replacement the labels for matching breaks.
-#' @param expand a numeric vector of length two, giving a multiplicative and
-#'   additive constant used to expand the range of the scales so that there
-#'   is a small gap between the data and the axes. The defaults are (0,0.6)
-#'   for discrete scales and (0.05,0) for continuous scales.
-#' @param na.value how should missing values be displayed?
-#' @param guide the name of, or actual function, used to create the
-#'   guide. See \code{\link{guides}} for more info.
+#'    The default, `TRUE`, uses the levels that appear in the data;
+#'    `FALSE` uses all the levels in the factor.
+#' @param na.translate Unlike continuous scales, discrete scales can easily show
+#'   missing values, and do so by default. If you want to remove missing values
+#'   from a discrete scale, specify `na.translate = FALSE`.
+#' @param na.value If `na.translate = TRUE`, what value aesthetic
+#'   value should missing be displayed as? Does not apply to position scales
+#'   where `NA` is always placed at the far right.
 #' @keywords internal
-discrete_scale <- function(aesthetics, scale_name, palette, name = waiver(), breaks = waiver(),
-  labels = waiver(), limits = NULL, expand = waiver(), na.value = NA, drop = TRUE,
-  guide = "legend") {
+discrete_scale <- function(aesthetics, scale_name, palette, name = waiver(),
+  breaks = waiver(), labels = waiver(), limits = NULL, expand = waiver(),
+  na.translate = TRUE, na.value = NA, drop = TRUE,
+  guide = "legend", position = "left", super = ScaleDiscrete) {
 
   check_breaks_labels(breaks, labels)
 
+  position <- match.arg(position, c("left", "right", "top", "bottom"))
+
   if (is.null(breaks) && !is_position_aes(aesthetics) && guide != "none") {
     guide <- "none"
   }
 
-  ggproto(NULL, ScaleDiscrete,
+  ggproto(NULL, super,
     call = match.call(),
 
     aesthetics = aesthetics,
@@ -620,12 +640,26 @@ discrete_scale <- function(aesthetics, scale_name, palette, name = waiver(), bre
     range = discrete_range(),
     limits = limits,
     na.value = na.value,
+    na.translate = na.translate,
     expand = expand,
 
     name = name,
     breaks = breaks,
     labels = labels,
     drop = drop,
-    guide = guide
+    guide = guide,
+    position = position
   )
 }
+
+# In place modification of a scale to change the primary axis
+scale_flip_position <- function(scale) {
+  scale$position <- switch(scale$position,
+    top = "bottom",
+    bottom = "top",
+    left = "right",
+    right = "left",
+    scale$position
+  )
+  invisible()
+}
