@@ -32,7 +32,7 @@ check_required_aesthetics <- function(required, present, name) {
 }
 
 # Concatenate a named list for output
-# Print a \code{list(a=1, b=2)} as \code{(a=1, b=2)}
+# Print a `list(a=1, b=2)` as `(a=1, b=2)`
 #
 # @param list to concatenate
 # @keyword internal
@@ -64,17 +64,17 @@ uniquecols <- function(df) {
 
 #' Convenience function to remove missing values from a data.frame
 #'
-#' Remove all non-complete rows, with a warning if \code{na.rm = FALSE}.
+#' Remove all non-complete rows, with a warning if `na.rm = FALSE`.
 #' ggplot is somewhat more accommodating of missing values than R generally.
 #' For those stats which require complete data, missing values will be
-#' automatically removed with a warning. If \code{na.rm = TRUE} is supplied
+#' automatically removed with a warning. If `na.rm = TRUE` is supplied
 #' to the statistic, the warning will be suppressed.
 #'
 #' @param df data.frame
 #' @param na.rm If true, will suppress warning message.
 #' @param vars Character vector of variables to check for missings in
 #' @param name Optional function name to improve error message.
-#' @param finite If \code{TRUE}, will also remove non-finite values.
+#' @param finite If `TRUE`, will also remove non-finite values.
 #' @keywords internal
 #' @export
 remove_missing <- function(df, na.rm = FALSE, vars = names(df), name = "",
@@ -85,10 +85,10 @@ remove_missing <- function(df, na.rm = FALSE, vars = names(df), name = "",
   if (name != "") name <- paste(" (", name, ")", sep = "")
 
   if (finite) {
-    missing <- !finite.cases(df[, vars, drop = FALSE])
+    missing <- !cases(df[, vars, drop = FALSE], is_finite)
     str <- "non-finite"
   } else {
-    missing <- !stats::complete.cases(df[, vars, drop = FALSE])
+    missing <- !cases(df[, vars, drop = FALSE], is_complete)
     str <- "missing"
   }
 
@@ -104,24 +104,40 @@ remove_missing <- function(df, na.rm = FALSE, vars = names(df), name = "",
   df
 }
 
-finite.cases <- function(x) UseMethod("finite.cases")
 # Returns a logical vector of same length as nrow(x). If all data on a row
 # is finite (not NA, NaN, Inf, or -Inf) return TRUE; otherwise FALSE.
-#' @export
-finite.cases.data.frame <- function(x) {
-  finite_cases <- vapply(x, is.finite, logical(nrow(x)))
+cases <- function(x, fun) {
+  ok <- vapply(x, fun, logical(nrow(x)))
 
   # Need a special case test when x has exactly one row, because rowSums
   # doesn't respect dimensions for 1x1 matrices. vapply returns a vector (not
   # a matrix when the input has one row.
-  if (is.vector(finite_cases)) {
-    all(finite_cases)
+  if (is.vector(ok)) {
+    all(ok)
   } else {
     # Find all the rows where all are TRUE
-    rowSums(as.matrix(finite_cases)) == ncol(x)
+    rowSums(as.matrix(ok)) == ncol(x)
   }
 }
 
+# Wrapper around is.finite to handle list cols
+is_finite <- function(x) {
+  if (typeof(x) == "list") {
+    !vapply(x, is.null, logical(1))
+  } else {
+    is.finite(x)
+  }
+}
+
+is_complete <- function(x) {
+  if (typeof(x) == "list") {
+    !vapply(x, is.null, logical(1))
+  } else {
+    !is.na(x)
+  }
+}
+
+
 #' Used in examples to illustrate when errors should occur.
 #'
 #' @param expr code to evaluate.
@@ -139,10 +155,10 @@ should_stop <- function(expr) {
 
 #' A waiver object.
 #'
-#' A waiver is a "flag" object, similar to \code{NULL}, that indicates the
+#' A waiver is a "flag" object, similar to `NULL`, that indicates the
 #' calling function should just use the default value.  It is used in certain
-#' functions to distinguish between displaying nothing (\code{NULL}) and
-#' displaying a default value calculated elsewhere (\code{waiver()})
+#' functions to distinguish between displaying nothing (`NULL`) and
+#' displaying a default value calculated elsewhere (`waiver()`)
 #'
 #' @export
 #' @keywords internal
@@ -156,25 +172,96 @@ rescale01 <- function(x) {
   (x - rng[1]) / (rng[2] - rng[1])
 }
 
+#' Similar to expand_range(), but taking a vector ‘expand’
+#' of *four* expansion values, where the 1st and 2nd
+#' elements are used for the lower limit, and the 3rd and
+#' 4th elements are used for the upper limit).
+#'
+#' The ‘expand’ argument can also be of length 2,
+#' and the expansion values for the lower limit
+#' are then reused for the upper limit.
+#
+#' @noRd
+#' @keywords internal
+expand_range4 <- function(limits, expand) {
+   stopifnot(is.numeric(expand) && (length(expand) %in% c(2,4)))
+   # If only two expansion constants are given (i.e. the old syntax),
+   # reuse them to generate a four-element expansion vector
+   if (length(expand) == 2) { expand <- c(expand, expand) }
+
+   # Calculate separate range expansion for the lower and
+   # upper range limits, and then combine them into one vector
+   lower <- expand_range(limits, expand[1], expand[2])[1]
+   upper <- expand_range(limits, expand[3], expand[4])[2]
+   c(lower, upper)
+}
+
+#' Generate expansion vector for scales.
+#'
+#' This is a convenience function for generating scale expansion vectors
+#' for the \code{expand} argument of
+#' \code{\link[=scale_x_continuous]{scale_*_continuous}} and
+#' \code{\link[=scale_x_discrete]{scale_*_discrete}}.
+#' The expansions vectors are used to add some space between
+#' the data and the axes.
+#'
+#' @export
+#' @param mult vector of multiplicative range expansion factors.
+#'   If length 1, both the lower and upper limits of the scale
+#'   are expanded outwards by \code{mult}. If length 2, the lower limit
+#'   is expanded by \code{mult[1]} and the upper limit by \code{mult[2]}.
+#' @param add vector of additive range expansion constants.
+#'   If length 1, both the lower and upper limits of the scale
+#'   are expanded outwards by \code{add} units. If length 2, the
+#'   lower limit is expanded by \code{add[1]} and the upper
+#'   limit by \code{add[2]}.
+#' @examples
+#' # No space below the bars but 10% above them
+#' ggplot(mtcars) +
+#'   geom_bar(aes(x = factor(cyl))) +
+#'   scale_y_continuous(expand = expand_scale(mult = c(0, .1)))
+#'
+#' # Add 2 units of space on the left and right of the data
+#' ggplot(subset(diamonds, carat > 2), aes(cut, clarity)) +
+#'   geom_jitter() +
+#'   scale_x_discrete(expand = expand_scale(add = 2))
+#'
+#' # Reproduce the default range expansion used
+#' # when the ‘expand’ argument is not specified
+#' ggplot(subset(diamonds, carat > 2), aes(cut, price)) +
+#'   geom_jitter() +
+#'   scale_x_discrete(expand = expand_scale(add = .6)) +
+#'   scale_y_continuous(expand = expand_scale(mult = .05))
+expand_scale = function(mult = 0, add = 0) {
+  stopifnot(is.numeric(mult) && is.numeric(add))
+  stopifnot((length(mult) %in% 1:2) && (length(add) %in% 1:2))
+
+  mult <- rep(mult, length.out = 2)
+  add <- rep(add, length.out = 2)
+  c(mult[1], add[1], mult[2], add[2])
+}
+
+
+
 #' Give a deprecation error, warning, or message, depending on version number.
 #'
 #' Version numbers have the format <major>.<minor>.<subminor>, like 0.9.2.
 #' This function compares the current version number of ggplot2 against the
-#' specified \code{version}, which is the most recent version before the
+#' specified `version`, which is the most recent version before the
 #' function (or other object) was deprecated.
 #'
-#' \code{gg_dep} will give an error, warning, or message, depending on the
+#' `gg_dep` will give an error, warning, or message, depending on the
 #' difference between the current ggplot2 version and the specified
-#' \code{version}.
+#' `version`.
 #'
-#' If the current major number is greater than \code{version}'s major number,
-#' or if the current minor number is more than 1 greater than \code{version}'s
+#' If the current major number is greater than `version`'s major number,
+#' or if the current minor number is more than 1 greater than `version`'s
 #' minor number, give an error.
 #'
-#' If the current minor number differs from \code{version}'s minor number by
+#' If the current minor number differs from `version`'s minor number by
 #' one, give a warning.
 #'
-#' If the current subminor number differs from \code{version}'s subminor
+#' If the current subminor number differs from `version`'s subminor
 #' number, print a message.
 #'
 #' @param version The last version of ggplot2 where this function was good
@@ -273,6 +360,15 @@ warning_wrap <- function(...) {
   warning(paste0(wrapped, collapse = "\n"), call. = FALSE)
 }
 
+var_list <- function(x) {
+  x <- encodeString(x, quote = "`")
+  if (length(x) > 5) {
+    x <- c(x[1:5], paste0("and ", length(x) - 5, " more"))
+  }
+
+  paste0(x, collapse = ", ")
+}
+
 dispatch_args <- function(f, ...) {
   args <- list(...)
   formals <- formals(f)
@@ -299,6 +395,38 @@ find_args <- function(...) {
 # global data
 dummy_data <- function() data.frame(x = NA)
 
+with_seed_null <- function(seed, code) {
+  if (is.null(seed)) {
+    code
+  } else {
+    withr::with_seed(seed, code)
+  }
+}
+
+seq_asc <- function(to, from) {
+  if (to > from) {
+    integer()
+  } else {
+    to:from
+  }
+}
+
 # Needed to trigger package loading
 #' @importFrom tibble tibble
 NULL
+
+# Check inputs with tibble but allow column vectors (see #2609 and #2374)
+as_gg_data_frame <- function(x) {
+  x <- lapply(x, validate_column_vec)
+  as.data.frame(tibble::as_tibble(x))
+}
+validate_column_vec <- function(x) {
+  if (is_column_vec(x)) {
+    dim(x) <- NULL
+  }
+  x
+}
+is_column_vec <- function(x) {
+  dims <- dim(x)
+  length(dims) == 2L && dims[[2]] == 1L
+}
