@@ -1,6 +1,5 @@
 context("Layer")
 
-
 # Parameters --------------------------------------------------------------
 
 test_that("aesthetics go in aes_params", {
@@ -16,25 +15,15 @@ test_that("unknown aesthietcs create warning", {
   expect_warning(geom_point(aes(blah = "red")), "unknown aesthetics")
 })
 
-# Calculated aesthetics ---------------------------------------------------
-
-test_that("Bare name surround by .. is calculated", {
-  expect_true(is_calculated_aes(aes(..density..)))
-  expect_true(is_calculated_aes(aes(..DENSITY..)))
-  expect_false(is_calculated_aes(aes(a..x..b)))
+test_that("unknown NULL asthetic doesn't create warning (#1909)", {
+  expect_warning(geom_point(aes(blah = NULL)), NA)
 })
 
-test_that("Calling using variable surround by .. is calculated", {
-  expect_true(is_calculated_aes(aes(mean(..density..))))
-  expect_true(is_calculated_aes(aes(mean(..DENSITY..))))
-  expect_false(is_calculated_aes(aes(mean(a..x..b))))
-})
-
-test_that("strip_dots remove dots around calculated aesthetics", {
-  expect_equal(strip_dots(aes(..density..))$x, quote(density))
-  expect_equal(strip_dots(aes(mean(..density..)))$x, quote(mean(density)))
-  expect_equal(strip_dots(aes(sapply(..density.., function(x) mean(x)))$x),
-               quote(sapply(density, function(x) mean(x))))
+test_that("column vectors are allowed (#2609)", {
+  df <- data.frame(x = 1:10)
+  df$y <- scale(1:10) # Returns a column vector
+  p <- ggplot(df, aes(x, y))
+  expect_is(layer_data(p), "data.frame")
 })
 
 # Data extraction ---------------------------------------------------------
